@@ -536,6 +536,19 @@ class Job:
         >>> job.status()
         <Status.Paid: 4>
 
+        >>> multi_credentials = [("0x61F9F0B31eacB420553da8BCC59DC617279731Ac", "486a0621e595dd7fcbe5608cbbeec8f5a8b5cabe7637f11eccfc7acd408c3a0e"), ("0x6b7E3C31F34cF38d1DFC1D9A8A59482028395809", "f22d4fc42da79aa5ba839998a0a9f2c2c45f5e55ee7f1504e464d2c71ca199e1")]
+        >>> job = Job(credentials, manifest, multi_credentials=multi_credentials)
+        >>> job.launch(rep_oracle_pub_key)
+        True
+        >>> job.setup()
+        True
+        >>> trusted_handlers = ['0x61F9F0B31eacB420553da8BCC59DC617279731Ac', '0x6b7E3C31F34cF38d1DFC1D9A8A59482028395809']
+        >>> job.add_trusted_handlers(trusted_handlers)
+        True
+        >>> payouts = [("0x6b7E3C31F34cF38d1DFC1D9A8A59482028395809", Decimal('20.0')), ("0x852023fbb19050B8291a335E5A83Ac9701E7B4E6", Decimal('50.0'))]
+        >>> job.bulk_payout(payouts, {}, rep_oracle_pub_key)
+        True
+
         Args:
             payouts (List[Tuple[str, int]]): a list of tuples with ethereum addresses and amounts.
             results (Dict): the final answer results stored by the Reputation Oracle.
@@ -545,20 +558,47 @@ class Job:
             bool: returns True if paying to ethereum addresses and oracles succeeds.
 
         """
-        (hash_, url) = upload(results, pub_key)
-        eth_addrs = [eth_addr for eth_addr, amount in payouts]
-        hmt_amounts = [int(amount * 10**18) for eth_addr, amount in payouts]
+        try:
+            (hash_, url) = upload(results, pub_key)
+            eth_addrs = [eth_addr for eth_addr, amount in payouts]
+            hmt_amounts = [
+                int(amount * 10**18) for eth_addr, amount in payouts
+            ]
 
-        txn_func = self.job_contract.functions.bulkPayOut
-        func_args = [eth_addrs, hmt_amounts, url, hash_, 1]
-        txn_info = {
-            "gas_payer": self.gas_payer,
-            "gas_payer_priv": self.gas_payer_priv,
-            "gas": gas
-        }
+            txn_func = self.job_contract.functions.bulkPayOut
+            func_args = [eth_addrs, hmt_amounts, url, hash_, 1]
+            txn_info = {
+                "gas_payer": self.gas_payer,
+                "gas_payer_priv": self.gas_payer_priv,
+                "gas": gas
+            }
 
-        handle_transaction(txn_func, *func_args, **txn_info)
-        return self._bulk_paid() == True
+            handle_transaction(txn_func, *func_args, **txn_info)
+            return self._bulk_paid() == True
+        except:
+            LOG.error(
+                f"Bulk payout failed with main credentials: {self.gas_payer}, {self.gas_payer_priv}"
+            )
+
+        bulk_paid = False
+
+        for gas_payer, gas_payer_priv in self.multi_credentials:
+            txn_info = {
+                "gas_payer": gas_payer,
+                "gas_payer_priv": gas_payer_priv,
+                "gas": gas
+            }
+            try:
+                handle_transaction(txn_func, *[], **txn_info)
+                self.gas_payer = gas_payer
+                self.gas_payer_priv = gas_payer_priv
+                bulk_paid = True
+                break
+            except:
+                LOG.error(
+                    f"Bulk payout failed with {gas_payer} and {gas_payer_priv}. Raffling new ones..."
+                )
+        return bulk_paid == True
 
     def abort(self, gas: int = GAS_LIMIT) -> bool:
         """Kills the contract and returns the HMT back to the gas payer.
@@ -723,6 +763,18 @@ class Job:
         >>> rep_oracle_priv_key = b"28e516f1e2f99e96a48a23cea1f94ee5f073403a1c68e818263f0eb898f1c8e5"
         >>> job.intermediate_results(rep_oracle_priv_key)
         {'results': True}
+        
+        >>> multi_credentials = [("0x61F9F0B31eacB420553da8BCC59DC617279731Ac", "486a0621e595dd7fcbe5608cbbeec8f5a8b5cabe7637f11eccfc7acd408c3a0e"), ("0x6b7E3C31F34cF38d1DFC1D9A8A59482028395809", "f22d4fc42da79aa5ba839998a0a9f2c2c45f5e55ee7f1504e464d2c71ca199e1")]
+        >>> job = Job(credentials, manifest, multi_credentials=multi_credentials)
+        >>> job.launch(rep_oracle_pub_key)
+        True
+        >>> job.setup()
+        True
+        >>> trusted_handlers = ['0x61F9F0B31eacB420553da8BCC59DC617279731Ac', '0x6b7E3C31F34cF38d1DFC1D9A8A59482028395809']
+        >>> job.add_trusted_handlers(trusted_handlers)
+        True
+        >>> job.store_intermediate_results(results, rep_oracle_pub_key)
+        True
 
         Args:
             results (Dict): intermediate results of the Recording Oracle.
@@ -732,17 +784,42 @@ class Job:
             returns True if contract's state is updated and IPFS upload succeeds.
 
         """
-        (hash_, url) = upload(results, pub_key)
-        txn_func = self.job_contract.functions.storeResults
-        func_args = [url, hash_]
-        txn_info = {
-            "gas_payer": self.gas_payer,
-            "gas_payer_priv": self.gas_payer_priv,
-            "gas": gas
-        }
+        try:
+            (hash_, url) = upload(results, pub_key)
+            txn_func = self.job_contract.functions.storeResults
+            func_args = [url, hash_]
+            txn_info = {
+                "gas_payer": self.gas_payer,
+                "gas_payer_priv": self.gas_payer_priv,
+                "gas": gas
+            }
 
-        handle_transaction(txn_func, *func_args, **txn_info)
-        return True
+            handle_transaction(txn_func, *func_args, **txn_info)
+            return True
+        except:
+            LOG.error(
+                f"Storing intermediate results failed with main credentials: {self.gas_payer}, {self.gas_payer_priv}"
+            )
+
+        results_stored = False
+
+        for gas_payer, gas_payer_priv in self.multi_credentials:
+            txn_info = {
+                "gas_payer": gas_payer,
+                "gas_payer_priv": gas_payer_priv,
+                "gas": gas
+            }
+            try:
+                handle_transaction(txn_func, *[], **txn_info)
+                self.gas_payer = gas_payer
+                self.gas_payer_priv = gas_payer_priv
+                results_stored = True
+                break
+            except:
+                LOG.error(
+                    f"Storing intermediate results failed with {gas_payer} and {gas_payer_priv}. Raffling new ones..."
+                )
+        return results_stored
 
     def complete(self, gas: int = GAS_LIMIT) -> bool:
         """Completes the Job if it has been paid.
@@ -1283,4 +1360,6 @@ class Job:
 if __name__ == "__main__":
     import doctest
     from test_manifest import manifest
+
+    # IMPORTANT, don't modify this so CI catches the doctest errors.
     doctest.testmod(raise_on_error=True)
